-- SPDX-FileCopyrightText: 2023 Technical University of Denmark
--
-- SPDX-License-Identifier: BSD-3-Clause

{- |
Module      : Main
Description : Test suite for noname.
Copyright   : 2023 Technical University of Denmark
License     : BSD-3-Clause
Maintainer  : lpkf@dtu.dk
Stability   : experimental
-}
module Main (main) where

-- bytestring
import Data.ByteString.Lazy (ByteString, fromStrict)

-- filepath
import System.FilePath (takeBaseName)

-- tasty
import Test.Tasty (TestTree, defaultMain, testGroup)

-- tasty-golden
import Test.Tasty.Golden (goldenVsString)

-- text
import Data.Text.Encoding (encodeUtf8)

-- noname
import Noname (Options (..), defaultOpts, nonameWithOpts)

-- | Run the golden tests.
main :: IO ()
main = defaultMain goldenTests

-- | The filepaths to the specifications for the golden tests.
nnFiles :: [FilePath]
nnFiles =
  [ "./examples/bac/bac.nn"
  , "./examples/bac/bac_fixed.nn"
  , "./examples/bac/bac_parallel.nn"
  , "./examples/bac/bac_sequential.nn"
  , "./examples/basic_hash/basic_hash.nn"
  , "./examples/basic_hash/basic_hash_compromise.nn"
  , "./examples/osk/osk_0_desynchro.nn"
  , "./examples/osk/osk_1_desynchro.nn"
  , "./examples/runex/runex.nn"
  , "./examples/runex/runex_fix_attempt.nn"
  , "./examples/runex/runex_fixed.nn"
  ]

{- | The golden tests: comparison between the expected value in the golden files
and the result of the protocol verifier.
-}
goldenTests :: TestTree
goldenTests =
  testGroup
    "Noname golden tests"
    [ goldenVsString name goldenFile $ nonameTest nnFile
    | nnFile <- nnFiles
    , let name = takeBaseName nnFile
    , let goldenFile = "./test/golden/" ++ name ++ ".golden"
    ]

-- | Wrapper around the protocol verifier function to be used in the tests.
nonameTest :: FilePath -> IO ByteString
nonameTest f =
  fromStrict . encodeUtf8
    <$> nonameWithOpts defaultOpts{optInput = Just f, optQuiet = True}
